
import { createRoot } from 'react-dom/client';
import { BrowserRouter } from 'react-router-dom';
import { QueryClient, QueryClientProvider } from '@tanstack/react-query';
import App from './App.tsx';
import './index.css';

// PWA Service Worker Registration for Healthcare Professionals
const registerServiceWorker = async () => {
  if ('serviceWorker' in navigator) {
    try {
      console.log('[PWA] Registering healthcare service worker...');
      
      // Force update check on registration
      const registration = await navigator.serviceWorker.register('/sw.js', {
        scope: '/',
        updateViaCache: 'none' // Disable cache for service worker script
      });
      
      console.log('[PWA] Healthcare service worker registered successfully:', registration.scope);
      
      // Force immediate update check
      registration.update();
      
      // Handle service worker updates for medical facility deployments
      registration.addEventListener('updatefound', () => {
        const newWorker = registration.installing;
        if (newWorker) {
          newWorker.addEventListener('statechange', () => {
            if (newWorker.state === 'installed' && navigator.serviceWorker.controller) {
              console.log('[PWA] New healthcare app version available');
              // The PWAUpdatePrompt component will handle user notification
            }
          });
        }
      });
      
      // Check for waiting service worker
      if (registration.waiting) {
        console.log('[PWA] Healthcare app update ready');
      }
      
      // Periodic update checks every 30 minutes for healthcare environments
      setInterval(() => {
        console.log('[PWA] Checking for app updates...');
        registration.update();
      }, 30 * 60 * 1000);
      
      return registration;
    } catch (error) {
      console.error('[PWA] Healthcare service worker registration failed:', error);
    }
  } else {
    console.log('[PWA] Service workers not supported in this medical facility browser');
  }
};

// PWA Install Prompt Management for Healthcare Professionals
const setupPWAInstallPrompt = () => {
  let deferredPrompt: any;
  
  // Listen for beforeinstallprompt event
  window.addEventListener('beforeinstallprompt', (e) => {
    console.log('[PWA] Healthcare app install prompt available');
    e.preventDefault();
    deferredPrompt = e;
    
    // Store the event for later use
    (window as any).deferredPrompt = deferredPrompt;
  });
  
  // Listen for app installed event
  window.addEventListener('appinstalled', () => {
    console.log('[PWA] Healthcare app installed successfully');
    deferredPrompt = null;
    
    // Track installation for healthcare analytics
    if (typeof (window as any).gtag !== 'undefined') {
      (window as any).gtag('event', 'pwa_install', {
        event_category: 'engagement',
        event_label: 'healthcare_professional'
      });
    }
  });
};

// PWA Criteria Checker for Healthcare Environment
const checkPWACriteria = () => {
  const criteria = {
    https: location.protocol === 'https:' || location.hostname === 'localhost',
    serviceWorker: 'serviceWorker' in navigator,
    manifest: document.querySelector('link[rel="manifest"]') !== null,
    standalone: window.matchMedia('(display-mode: standalone)').matches,
    ios: /iPad|iPhone|iPod/.test(navigator.userAgent)
  };
  
  console.log('[PWA] Healthcare app criteria check:', criteria);
  
  if (criteria.https && criteria.serviceWorker && criteria.manifest) {
    console.log('[PWA] Healthcare app meets PWA requirements');
  } else {
    console.warn('[PWA] Healthcare app missing PWA requirements:', {
      needsHTTPS: !criteria.https,
      needsServiceWorker: !criteria.serviceWorker,
      needsManifest: !criteria.manifest
    });
  }
  
  return criteria;
};

// Initialize PWA features for healthcare professionals
const initializePWA = async () => {
  console.log('[PWA] Initializing healthcare professional app...');
  
  // Check PWA criteria
  const criteria = checkPWACriteria();
  
  // Register service worker (non-blocking)
  const registration = await registerServiceWorker().catch(console.error);
  
  // Setup install prompt
  setupPWAInstallPrompt();
  
  // Initialize cache management
  if (registration) {
    // Clear old caches on startup to prevent accumulation
    try {
      const { clearOldCaches } = await import('./utils/cacheUtils');
      await clearOldCaches();
    } catch (error) {
      console.warn('[PWA] Cache cleanup failed:', error);
    }
  }
  
  // Platform-specific optimizations for medical devices
  if (criteria.ios) {
    console.log('[PWA] iOS medical device detected - optimizing for healthcare workflow');
    // Add iOS-specific meta tags if needed
    const metaAppleCapable = document.querySelector('meta[name="apple-mobile-web-app-capable"]');
    if (!metaAppleCapable) {
      const meta = document.createElement('meta');
      meta.name = 'apple-mobile-web-app-capable';
      meta.content = 'yes';
      document.head.appendChild(meta);
    }
  }
  
  // Check if already installed
  if (criteria.standalone) {
    console.log('[PWA] Healthcare app running in standalone mode');
  }
  
  console.log('[PWA] Healthcare app PWA initialization complete');
};

// Initialize PWA after initial render (non-blocking)
if (typeof window !== 'undefined') {
  // Defer PWA initialization to avoid blocking initial render
  const deferredPWAInit = () => {
    if ('requestIdleCallback' in window) {
      requestIdleCallback(() => initializePWA(), { timeout: 2000 });
    } else {
      setTimeout(initializePWA, 100);
    }
  };
  
  if (document.readyState === 'complete') {
    deferredPWAInit();
  } else {
    window.addEventListener('load', deferredPWAInit);
  }
}

// Create React app with healthcare optimizations
const queryClient = new QueryClient({
  defaultOptions: {
    queries: {
      staleTime: 5 * 60 * 1000, // 5 minutes for healthcare data
      retry: 3, // Retry failed requests (important for medical facilities)
      refetchOnWindowFocus: true, // Refresh data when healthcare professional returns to app
    },
  },
});

createRoot(document.getElementById("root")!).render(
  <BrowserRouter>
    <QueryClientProvider client={queryClient}>
      <App />
    </QueryClientProvider>
  </BrowserRouter>
);
